"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

const fs = require(`fs`);

const util = require(`../util/tool.js`);

function getHtml(data) {
  var _context, _context2, _context3;

  // 文件路径
  const pathList = (0, _map.default)(_context = data.path.replace(/\/$/, ``).split(`/`)).call(_context, (item, index, arr) => {
    const cur = (0, _slice.default)(arr).call(arr, 0, index + 1);
    return {
      cur,
      href: `${data.baseUrl}${cur.join(`/`) || `/`}`,
      text: `/${(0, _slice.default)(cur).call(cur, -1)[0]}`
    };
  });
  const pathStr = (0, _reduce.default)(pathList).call(pathList, (acc, cur) => `${acc}<a href="${cur.href}">${cur.text}</a>`, ``); // 文件列表

  const fileListStr = (0, _reduce.default)(_context2 = (0, _map.default)(_context3 = data.files).call(_context3, item => ({ ...item,
    mtime: util.time.dateFormat(`YYYY-MM-DD hh:mm:ss`, new Date(item.mtime)),
    name: item.isFile ? item.name : `${item.name}/`,
    size: item.isFile ? util.hex.getSize(item.size) : `--`
  }))).call(_context2, (acc, cur) => {
    return `${acc}
        <a href="./${cur.name}">
          <div class="name">${cur.name}</div>
          <div class="size">${cur.size}</div>
          <div class="mtime">${cur.mtime}</div>
        </a>
      `;
  }, pathList.length > 1 ? `
        <a href="..">
          <div class="name">..</div>
          <div class="size">--</div>
          <div class="mtime">--</div>
        </a>
      ` : ``); // 文件模板

  const template = `
    <head>
      <base href="${data.originalUrl}" />
    </head>
    <style>
      html,
      body {
        margin: 0;
        padding: 0;
      }
      .main {
        font-family: 'Courier New', Courier, monospace;
      }
      .main .path a:hover {
        background-color: #eee;
      }
      .main .list.header {
        font-weight: bold;
      }
      .main .list a {
        display: block;
        text-decoration: none;
      }
      .main .list a:hover {
        background-color: #eee;
      }
      .main .list .name,
      .main .list .size,
      .main .list .mtime {
        display: inline-block;
        overflow: hidden;
        text-overflow: ellipsis;
        white-space: nowrap;
      }
      .main .list .name {
        width: calc(100% - 330px);
      }
      .main .list .size {
        width: 100px;
        text-align: right;
      }
      .main .list .mtime {
        width: 200px;
      }
    </style>
    <div class="main">
      <div class="path">${pathStr}</div>
      <div class="list header">
        <div>
          <div class="name">name</div>
          <div class="size">size</div>
          <div class="mtime">mtime</div>
        </div>
      </div>
      <div class="list fileList">${fileListStr}</div>
    </div>
  `;
  return template;
}

const obj = {
  post() {},

  delete() {},

  /**
   * 获取目录下的内容列表
   * @param {*} option
   * @param {*} option.root 目录地址
   * @param {*} option.sort 排序的 key
   * @param {*} option.order 排序方式 asc desc
   * @returns
   */
  async get(option) {
    var _context4, _context5;

    const list = (0, _filter.default)(_context4 = (0, _map.default)(_context5 = fs.readdirSync(option.root)).call(_context5, name => {
      try {
        const stat = fs.statSync(`${option.root}/${name}`);
        const isFile = stat.isFile();
        const isDirectory = stat.isDirectory();
        return isFile || isDirectory ? {
          name,
          // 文件大小
          size: stat.size,
          // 修改时间
          mtime: stat.mtime,
          // 创建时间
          birthtime: stat.birthtime,
          // 是否是文件
          isFile // stat,

        } : undefined;
      } catch (error) {
        return undefined;
      }
    })).call(_context4, item => item);
    return list;
  },

  put() {}

};

module.exports = option => {
  return async (req, res, next) => {
    const url = decodeURI(req.url.split(`?`)[0]);
    const path = `${option.root}/${url}`;

    if (req.method.toLowerCase() === `get`) {
      if (fs.existsSync(path)) {
        if (fs.statSync(path).isDirectory()) {
          const files = await obj.get({
            root: path
          });
          const data = {
            baseUrl: req.baseUrl,
            originalUrl: `${req.originalUrl}/`.replace(/\/\/$/, `/`),
            path: url,
            files
          };
          res.send(getHtml(data));
        } else {
          res.sendFile(path, {
            hidden: true
          });
        }
      } else {
        res.status(404);
        res.json({
          msg: `no such file or directory ${path}`
        });
      }
    }
  };
};